﻿/*
 MoveFolder.js
 ver 2.0		2011/05/07
 ・機能
 ニコニコ動画内でそのページにあるリンク（サムネ有りのみ）のうち、cacheがあるものを表示
 →個別 or 一括でフォルダ移動可能 & cacheファイル名変更可能
 
 ・使い方
 localフォルダにこのjsファイルをおいて、以下のnlFilterを導入
 
 
[Replace]
Name = topBar FolderMove.jsリンク追加(watch以外)
URL = www.nicovideo.jp/(?!watch)
Match<
<span id="nlbar">
>
Replace<
$0 \|
<a href="javascript:void(0)" onclick="NicoCacheMove.togglePanel();">Move</a>
<script type="text/javascript" charset="utf-8" src="/local/MoveFolder.js?_2011_04_01"></script>
<style type="text/css">
tr.NCM_checked { background-color: LightCyan; }
</style>
>
 
 
 ・注意事項
 初回のFlvlistの受信は時間がかかります。２回目以降（ページを移動しても）はすぐ受信できるので快適です。
 一度表示された後は、「Folder」をクリックしてもパネルの表示/非表示の切り替えのみ
 mylistやAutoPagerでの再取得(表示動画を変えた場合)は[更新]をクリック
 タイトル部分を右クリックでcacheファイル名変更可能
 
 //*/
(function () {
	window.NicoCacheMove = {
		//	全キャッシュの情報を受信してキャッシュの有無を確認するかどうか	falseならajax_infoでそれぞれ取得
		//	※falseの場合ページ内サムネの数が多ければ多いほど遅くなりますが、
		//		キャッシュがあまりにも多い人は、全キャッシュの情報を受信よりは早いかもしれません。
		//			ただし、非同期でやるので、cacheの有無の確認が終わるまでページの操作ができません。
		//	※ useFlvlist:true 推奨
		useFlvlist: true,
		//デフォルトのcacheフォルダの名前（表示のみ・デフォルトだと空白[""]でわかりにくいため）
		defaultFolderName: '#cache',

		size: function(){
			return Math.round((window.innerHeight || document.documentElement.clientHeight || document.body.clientHeight) * 90 / 100);
		},
		//操作用パネル表示/非表示
		togglePanel: function () {
			if (this.withCacheSearch) {
				if (this.ControlPanel && $(this.ControlPanel).visible()){
					$(this.ControlPanel).hide();
				} else {
					if (this.data && this.data.dirList) {
						this.getCached();
						this.makePanel();
					} else {
						this.load();
					}
				}
				return;
			}
			if (this.ControlPanel) {
				$(this.ControlPanel).toggle();
				this.resize();
			} else {
				if (this.data && this.data.dirList) {
					this.getCached();
					this.makePanel();
				} else {
					this.load();
				}
			}
		},
		showThumbImg: function (id, n, e) {
			var el = $(this.table).getElementsByClassName('NCM_Img')[n];
			if(el.src.match(/res/)) el.src = 'http://tn-skr1.smilevideo.jp/smile?i=' + id;
			e = e || window.event;
			el.style.left = e.clientX - 135 + 'px';
			el.style.top = ((e.clientY - 100 < 0) ? e.clientY : e.clientY - 100 ) + 'px';
			el.show();
		},
		//移動
		move: function (smid, only_top, elm, byALL) {
			var self = this;
			var folder_name = this.data.dirList[this.folderNow];
			if (only_top) {
				var url = '/cache/ajax_topmove';
			} else {
				var url = '/cache/ajax_move';
			}
			new Ajax.Request(url, {
				method: 'GET',
				parameters: smid + '-' + encodeURI(folder_name),
				asynchronous: true,
				onComplete: function (xhr) {
					if (xhr.responseText == 'OK') {
						if (folder_name == '') {
							folder_name = self.defaultFolderName;
						}
						if (self.useFlvlist) {
							// パネルの表示を更新
							var nextElm_Child_P = elm.parentNode.nextSibling.getElementsByTagName('p')[0];
							nextElm_Child_P.title = folder_name;
							nextElm_Child_P.replaceChild(document.createTextNode(folder_name), nextElm_Child_P.firstChild);
							// Flvlistの情報を更新
							self.updateCacheList(smid, 1, folder_name);
						}
						var text = '[' + folder_name + ']に移動しました';
					} else {
						var text = '失敗しました(cache直下になかった等)';
					}
					elm.value = xhr.responseText;
					elm.title = text;
					// 表示データ更新
					if (NicoCacheMove.withCacheSearch) {
						if (byALL) {
							self.allMove_count++;
							if (self.num_nowSeleted === self.allMove_count) {
								window.updateShowingMyList();
							}
						} else {
							window.updateShowingMyList();
						}
					}
				}
			});
		},
		allMove_count:0,
		//全移動
		allMove: function (only_top) {
			var smid = this.smid_cached;
			var check = $(this.table).getElementsByClassName('NCM_checkbox');
			var button = $(this.table).getElementsByClassName('NCM_moveButton');
			var byALL = true;
			this.allMove_count = 0;
			for (var i = 0, len = smid.length; i < len; ++i) {
				if(check[i].checked) {
					this.move(smid[i][0], only_top, button[i], byALL);
				}
			}
		},
		getData: function (func) {
			//キャッシュディレクトリなどのデータ
			this.data = {};
			var self = this;
			if (this.useFlvlist && !this.withCacheSearch) {
				new Ajax.Request('/cache/flvlist.json', {
					method: 'GET',
					asynchronous: true,
					onComplete: function (xhr, json) {
						self.data = (json) ? json : xhr.responseText.evalJSON();
						if(typeof func === 'function') func();
					}
				});
			} else {
				new Ajax.Request('/cache/dirlist.json', {
					method: 'GET',
					asynchronous: true,
					onComplete: function (xhr, json) {
						self.data.dirList = (json) ? json : xhr.responseText.evalJSON();
						if(typeof func === 'function') func();
					}
				});
			}
		},
		// Flvlistの情報を更新(Flvlist使用時)
		updateCacheList: function (smid, num, newData) {
			if (NicoCacheMove.withCacheSearch) {
				// window.updateDataList : cacheSearch.htmlに存在
				if (num === 0) {
					window.updateDataList(smid, "title", newData);
					window.updateShowingMyList();
				} else if (num === 1) {
					window.updateDataList(smid, "folder", newData);
					window.updateDataList(smid, "description", newData);
				}
				return;
			}
			if (this.useFlvlist) {
				var cacheList = this.data.cacheList;
				if (cacheList[smid]) {
					cacheList[smid][num] = newData;
				} else {
					cacheList[smid + 'low'][num] = newData;
				}
			}
		},
		//ページ内にあるsmid(サムネのリンクの方)の配列
		smid_inPage: [],
		//ページ内にあるsmid(サムネのリンクの方)の文字列 すでにあるかの確認用
		smid_inPage_Str: ',',
		getLink: function () {
			var elms = document.getElementsByTagName('a');
			for (var i = 0, len = elms.length; i < len; ++i) {
				var elm_a = elms[i];
				if (elm_a.getElementsByTagName('img').length && elm_a.href && elm_a.href.match(/watch\/([a-z]{2}\d+)/)) {
					if (!this.smid_inPage_Str.match(',' + RegExp.$1 + ',')) {
						this.smid_inPage.push(RegExp.$1);
						this.smid_inPage_Str += RegExp.$1 + ',';
					}
				}
			}
		},
		// cacheSearch.htmlで仕様中か否か
		withCacheSearch: false,
		getCached: function (all) {
			//キャッシュ済みでページ内にあるsmidの配列
			this.smid_cached = [];
			if (this.useFlvlist && all) {
				var cacheList = this.data.cacheList;
				for(var key in cacheList) {
					this.smid_cached.push([key, cacheList[key][0], cacheList[key][1]]);
				}
				return;
			} 
			if (NicoCacheMove.withCacheSearch) {
				this.smid_inPage = [];
				var items = my.currentItems;
				for (var i = 0, len = items.length; i< len; i++){
					// description = folder
					this.smid_cached.push([items[i].watch_id, items[i].title, items[i].description]);
				}
				return;
			}
			this.getLink();
			var smid = this.smid_inPage;
			if (smid.length) {
				if (this.useFlvlist) {
					var cacheList = this.data.cacheList;
					for (var i = 0, len = smid.length; i < len; ++i) {
						var List = (cacheList[smid[i]] || cacheList[smid[i] + 'low']);
						if (List) {
							this.smid_cached.push([smid[i], List[0], List[1]]);
						}
					}
				} else {
					var self = this;
					for (var i = 0, len = smid.length; i < len; ++i) {
						new Ajax.Request('/cache/ajax_info?' + smid[i], {
							method: 'GET',
							asynchronous: false,
							onComplete: function (xhr) {
								if (xhr.responseText && xhr.responseText.match(/^OK(?:,[^,]*){3},0,(.*)$/)) {
									self.smid_cached.push([smid[i], RegExp.$1]);
								}
							}
						});
					}
				}
			}
		},
		//操作用パネル作成
		makePanel: function () {
			if (this.ControlPanel) {
				Element.remove($(this.ControlPanel));
			}
			this.ControlPanel = MakeElement(document.body, ['div', [
				['style.display', 'block'],
				['style.zIndex', '15000'],
				['style.position', 'fixed'],
				['style.top', '50%'],
				['style.left', '50%'],
				['style.marginTop', '-' + this.size() / 2 + 'px'],
				['style.marginLeft', '-45%'],
				['style.height', this.size() + 'px'],
				['style.width', '90%'],
				['style.background', 'white'],
				['style.border', 'solid 2px black']
			]]);
			this.appendPanelTop();
		},
		num_nowSeleted:0,
		allMoveButtonUpdate: function(){
			var n = 0;
			var elm = $(this.table).getElementsByClassName('NCM_checkbox');
			for(var i = 0, len = elm.length; i < len; i++){
				if(elm[i].checked) n++;
			}
			this.allMoveButton.value = '選択中 (' + n + '個) を移動';
			this.num_nowSeleted = n;
		},
		appendPanelTop: function () {
			if (this.top) {
				Element.remove($(this.top));
			}
			//フォルダ移動先セレクトボックスでの選択中
			this.folderNow = 0;
			var smid = this.smid_cached;
			var self = this;
			var top = document.createElement('div');

			this.allMoveButton = MakeElement(top, ['input', [
				['type', 'button'],
				['value', '以下の選択中動画 (' + this.smid_cached.length + '個) を移動'],
				['title', '左クリックで全キャッシュフォルダ対象 \r\n/ 右クリックでcache直下のファイルのみ対象'],
				['onclick', function () {
					self.allMove(false);
				}],
				['oncontextmenu', function () {
					self.allMove(true);
					return false;
				}]
			]]);
			var folderSelect = MakeElement(top, ['select', [
				["title", 'すべての移動で、このセレクトボックスでの選択中のフォルダへ移動します。'],
				['onchange', function () {
					self.folderNow = this.value;
				}]
			]]);
			for (var i = 0, len = this.data.dirList.length; i < len; ++i) {
				MakeElement(folderSelect, ['option', ["value", String(i)],
					[true, (i ? this.data.dirList[i] : this.defaultFolderName)]
				]);
			}
			top.appendChild(folderSelect);

			//	閉じるボタン	一番右上に置くために最初にfloat:right
			//	AllCacheボタン	useFlvlist使用時
			//	更新ボタン
			this.top = MakeElement(this.ControlPanel, [top, [],
				['input', [
					['type', 'button'],
					['value', '閉じる'],
					[(isIE ? 'style.styleFloat' : 'style.cssFloat'), 'right'],
					["onclick", function () {
						$(self.ControlPanel).hide();
					}]
				]],
				[this.useFlvlist && !this.withCacheSearch ? 'input' : 0, [
					['type', 'button'],
					['value', 'AllCache'],
					['title', 'すべてのキャッシュを表示\n※注意！激重'],
					[(isIE ? 'style.styleFloat' : 'style.cssFloat'), 'right'],
					["onclick", function () {
						self.getCached(true);
						self.appendPanelTable();
						self.allMoveButtonUpdate();
					}]
				]],
				[!this.withCacheSearch ? 'input' : 0, [
					['type', 'button'],
					['value', '更新'],
					['title', 'クリック=ページ内の情報に対して更新。\r\n右クリック=NicoCacheの[キャッシュ情報・ディレクトリ情報]も更新。'],
					[(isIE ? 'style.styleFloat' : 'style.cssFloat'), 'right'],
					["onclick", function () {
						self.getCached();
						self.appendPanelTable();
						self.allMoveButtonUpdate();
					}],
					["oncontextmenu", function(){
						self.getData(function () {
							self.getCached();
							self.appendPanelTop();
						});
						return false;
					}]
				]]
			]);
			this.appendPanelTable();
			this.allMoveButtonUpdate();
		},
		appendPanelTable: function () {
			var self = this;
			if (this.table) {
				Element.remove($(this.table));
			}
			var smid = this.smid_cached;


			var tbody = document.createElement('tbody');
			this.table = MakeElement(this.ControlPanel, ['div', [
				['style.overflow', 'auto'],
				['style.height', $(this.ControlPanel).offsetHeight - $(this.top).offsetHeight - 5 + 'px']
			],
				['table', ['style.tableLayout', 'fixed', 'style.overflow', 'auto'],
					[tbody]
				]
			]);

			var help = MakeElement(tbody, ['tr', [],
				['td', ['style.textAlign', 'center', 'title', '全切り替え\r\nShift押しながらで一括選択できます。'],
					['input', [
						['type', 'checkbox'],
						['checked', true],
						['onclick', function(){
							var check = this.checked;
							$A($(self.table).getElementsByClassName('NCM_checkbox')).each(function(box){
								box.checked = check;
								box.parentNode.parentNode.className = check ? "NCM_checked" : "";
							});
							self.allMoveButtonUpdate();
						}]
					]]
				],
				['td', [
					[true, '個別'],
					['title', '個別移動用ボタン\r\n移動成功するとOK/失敗するとNGを表示\r\n右クリックした場合はcacheフォルダ直下にある場合のみ移動'],
					['style.textAlign', 'center']
				]],
				[this.useFlvlist ? 'td' : 0, [
					[true, '　　保存フォルダ　　'],
					["width", '20%'],
					['style.whiteSpace', 'pre'],
					['style.textAlign', 'center'],
					['title', '動画の現在の保存フォルダ・規定の幅を超えた分は非表示\r\ntitle属性に全文あるのでマウスオーバーで表示可能']
				]],
				[this.useFlvlist ? 'td' : 0, [true, '：']],
				['td', [
					[true, '動画ID'],
					['title', 'cache済み動画のsmid(マウスオーバーでサムネ表示)'],
					['style.textAlign', 'center']
				]],
				['td', [true, '：']],
				['td', [
					[true, 'cacheタイトル (右クリックでファイル名変更)'],
					['title', 'cacheファイルのタイトル≠現在のニコニコでのタイトル']
				]]
			]);

			$A(help.getElementsByTagName('td')).each(function (obj) {
				if (obj.title) {
					obj.style.border = 'solid 1px #333';
				}
			});

			function make(n, smid, title, folder) {
				folder = folder || self.defaultFolderName;
				var elem = MakeElement(false, ['p', [
					['style.whiteSpace', 'pre'],
					['title', title]
				],
					[true, title]
				]);
				//	ここからタイトル変更用
				var titleChange = function () {
					var input = document.createElement('input');
					input.type = 'text';
					input.value = title;
					input.title = '空欄でデフォルト';
					input.style.width = elem.parentNode.offsetWidth + 'px';
					input.style.border = 'solid 1px black';
					input.onkeypress = function (evt) {
						evt = window.event || evt;
						if (evt.keyCode == 13 || evt.which == 13) {
							this.onblur();
						}
					};
					input.onblur = function () {
						this.onblur = null;
						var new_title = elem.firstChild.value;
						if (new_title != title) {
							new Ajax.Request("/cache/ajax_title?" + smid + (new_title != "" ? '-' + encodeURIComponent(new_title) : ""), {
								method: 'GET',
								asynchronous: false,
								onComplete: function (xhr) {
									title = xhr.responseText.match(/^OK (.+)$/) ? RegExp.$1 : "null";
									elem.innerHTML = title;
									// Flvlistの情報を更新
									self.updateCacheList(smid, 0, title);
								}
							});
						} else {
							elem.innerHTML = title;
						}
						elem.oncontextmenu = function () {
							titleChange();
							return false;
						}
					};
					elem.oncontextmenu = function () {};
					elem.replaceChild(input, elem.firstChild);
					elem.firstChild.focus();
					elem.firstChild.select();
					return false;
				};
				elem.oncontextmenu = titleChange;
				//	ここまで

				MakeElement(tbody, ['tr', ["className", "NCM_checked"],
					['td', ['style.textAlign', 'center'],
						['input', [
							['type', 'checkbox'],
							['className', 'NCM_checkbox'],
							['title', '一括移動時に移動する動画'],
							['checked', true],
							['onclick', function(){
								var check = this.checked;
								if(self.shiftDownNow && typeof self.last_checked == 'number'){
									if(self.last_checked > n) {
										var i = n;
										var e = self.last_checked;
									} else {
										var i = self.last_checked;
										var e = n;
									}
									var box = $(self.table).getElementsByClassName('NCM_checkbox');
									for (i++; i < e; i++){
										box[i].checked = check;
										box[i].parentNode.parentNode.className = check ? "NCM_checked" : "";
									}
								}
								this.parentNode.parentNode.className = check ? "NCM_checked" : "";
								self.last_checked = n;
								self.allMoveButtonUpdate();
							}]
						]]
					],
					['td', ['style.textAlign', 'center'],
						['input', [
							['type', 'button'],
							['value', '移動'],
							['className', 'NCM_moveButton'],
							['title', '左クリックで全キャッシュフォルダ対象 \r\n/ 右クリックでcache直下のファイルのみ対象'],
							['onclick', function () {
								self.move(smid, false, this);
							}],
							['oncontextmenu', function () {
								self.move(smid, true, this);
								return false;
							}]
						]]
					],
					[self.useFlvlist ? 'td' : 0, ['width', '20%'],
						['p', [
							[true, folder],
							['style.height', '1.2em'],
							['style.overflow', 'hidden'],
							['style.wordBreak', 'break-all'],
							['title', folder]
						]]
					],
					[self.useFlvlist ? 'td' : 0, [true, '：']],
					['td', ['style.textAlign', 'right'],
						['a', [
							[true, smid],
							['href', 'http://www.nicovideo.jp/watch/' + smid],
							['onmouseout', function(){
								$(self.table).getElementsByClassName('NCM_Img')[n].hide();
							}],
							['onmouseover', function (e) {
								self.showThumbImg(smid.match(/\d+/), n, e);
							}]
						]],
						['img', [
							['src', 'http://res.nimg.jp/img/_.gif'],
							['className', 'NCM_Img',],
							['style.display', 'none'],
							['style.position', 'fixed'],
							['style.border', 'solid 2px #333'],
							['width', '130'],
							['height', '100']
						]]
					],
					['td', [true, '：']],
					['td', [],
						[elem]
					]
				]);
			}
			for (var j = 0, len = smid.length; j < len; ++j) {
				make(j, smid[j][0], smid[j][1], smid[j][2]);
			}
		},
		resize: function(){
			var CPanel = $(this.ControlPanel);
			if(!CPanel || !CPanel.offsetHeight) return;
			CPanel.style.marginTop = '-' + this.size() / 2 + 'px';
			CPanel.style.height = this.size() + 'px';
			this.table.style.height = CPanel.offsetHeight - $(this.top).offsetHeight - 5 + 'px';
		},
		load: function () {
			var self = this;
			this.getData(function () {
				self.getCached();
				self.makePanel();
			});
		},
		__empty: function () {}
	}

	//	↓jsをload時にflvList.jsonを受信。無駄な気がするのでコメントアウトしてあります。
	//NicoCacheMove.getData();


	Event.observe(window, 'resize', function(){
		NicoCacheMove.resize();
	});


	/*	要素追加用関数(elm「親要素」, arg[追加要素, 属性, 子要素(argと同じ形式・複数可能)])
	 親要素はID名かエレメントを指定。
	 返り値は追加した要素（親直下・arg[0]）
	 親要素elmをfalseにすると追加ではなく作成のみを行い、それを返す
	 追加要素は"div"や"p"などの文字列でも、すでにあるエレメントでもok
	 追加要素をtrueに指定すると、文字列をcreateTextNodeして追加。(elm, [true, 'test'])
	 属性部分でも文字列ではなく、trueにするとcreateTextNodeで追加(elm, ['p', [true, 'test']])
	 属性部分は２個づつ配列でくくっても、くくらなくてもOK
	 http://jsbeautifier.org/?tabsize=1で整形したときに見やすくなるようになってます。
	 */
	function MakeElement(elm, arg) {
		if(elm === 0) return;
		try {
			var tag = arg[0];
			var attr = arg[1];
			var child = arg.slice(2);
			elm = (typeof elm === 'string') ? document.getElementById(elm) : elm;
			if (tag) {
				if (tag === true) {
					attr = String(attr);
					elm.appendChild(document.createTextNode(attr));
				} else {
					tag = (typeof tag == 'string') ? document.createElement(tag) : tag;
						if (attr && attr.length) {
							if (typeof attr[0] == 'string' || attr[0] === true) {
								for (var i = 1, len = attr.length; i < len; i = i + 2) {
									if (attr[i - 1] === true) {
										attr[i] = String(attr[i]);
										tag.appendChild(document.createTextNode(attr[i]));
									} else {
										eval('tag.' + attr[i - 1] + '=attr[i]');
									}
								}
							} else {
								for (var i = 0, len = attr.length; i < len; i++) {
									if (attr[i][0] === true) {
										tag.appendChild(document.createTextNode(attr[i][1]));
									} else {
										eval('tag.' + attr[i][0] + '=attr[i][1]');
									}
								}
							}
						}
					if(elm){
						elm = elm.appendChild(tag);
					} else {
						elm = tag;
					}
					if (child && child.length) {
						for (var n = 0, len = child.length; n < len; n++) {
							MakeElement(elm, child[n]);
						}
					}
				}
			}
			return elm;
		} catch (e) {
			alert('MakeElement' + e);
		}
	}
	var isIE = Prototype.Browser.IE;
	
	Event.observe(document.body, 'keydown', function(evt){
		evt = evt || window.event;
		if(evt.keyCode === 16 || evt.which === 16){
			NicoCacheMove.shiftDownNow = true;
		}
	});
	Event.observe(document.body, 'keyup', function(){
		NicoCacheMove.shiftDownNow = false;
	});
	Event.observe(window, 'blur', function(){
		NicoCacheMove.shiftDownNow = false;
	});
	NicoCacheMove.shiftDownNow = false;
})();